package ru.ac.uniyar

import org.http4k.core.Body
import org.http4k.core.ContentType
import org.http4k.core.Filter
import org.http4k.core.HttpHandler
import org.http4k.core.Method.GET
import org.http4k.core.Response
import org.http4k.core.Status.Companion.BAD_REQUEST
import org.http4k.core.Status.Companion.FOUND
import org.http4k.core.Status.Companion.OK
import org.http4k.core.then
import org.http4k.filter.DebuggingFilters.PrintRequest
import org.http4k.lens.BiDiBodyLens
import org.http4k.routing.ResourceLoader
import org.http4k.routing.bind
import org.http4k.routing.path
import org.http4k.routing.routes
import org.http4k.routing.static
import org.http4k.server.Undertow
import org.http4k.server.asServer
import org.http4k.template.PebbleTemplates
import org.http4k.template.TemplateRenderer
import org.http4k.template.ViewModel
import org.http4k.template.viewModel
import ru.ac.uniyar.domain.Schedule
import ru.ac.uniyar.domain.WorkingHours
import ru.ac.uniyar.models.ShowErrorInfoViewModel
import ru.ac.uniyar.models.ShowScheduleViewModel
import ru.ac.uniyar.models.ShowWorkingHoursViewModel
import scheduleCreationRoute
import java.time.LocalDate
import java.time.LocalTime

fun app(
    renderer: TemplateRenderer,
    htmlView: BiDiBodyLens<ViewModel>,
    schedule: Schedule,
): HttpHandler = routes(
    "/ping" bind GET to pingHandler(),
    "/" bind GET to redirectToSchedule(),
    "/schedule" bind GET to showSchedule(schedule, renderer),
    "/schedule/new" bind scheduleCreationRoute(schedule, htmlView),
    "/schedule/{number}" bind GET to showWorkingHours(schedule, renderer),
    static(ResourceLoader.Classpath("/ru/ac/uniyar/public/")),
)

fun pingHandler(): HttpHandler = {
    Response(OK).body("pong")
}

fun redirectToSchedule(): HttpHandler = {
    Response(FOUND).header("Location", "/schedule")
}

fun showSchedule(schedule: Schedule, renderer: TemplateRenderer): HttpHandler = {
    val model = ShowScheduleViewModel(schedule.listWorkingHours())
    Response(OK).body(renderer(model))
}

fun showWorkingHours(schedule: Schedule, renderer: TemplateRenderer): HttpHandler = handler@{ request ->
    val indexString = request.path("number").orEmpty()
    val index = indexString.toIntOrNull() ?: return@handler Response(BAD_REQUEST)
    val hours = schedule.fetch(index) ?: return@handler Response(BAD_REQUEST)
    Response(OK).body(renderer(ShowWorkingHoursViewModel(hours)))
}

fun createSchedule(): Schedule {
    val schedule = Schedule()
    schedule.add(WorkingHours(
        LocalDate.of(2022, 10, 15),
        LocalTime.of(12, 0,0),
        LocalTime.of(18, 30, 0),
        "Обычный рабочий день",
    ))
    return schedule
}

fun showErrorMessageFilter(renderer: TemplateRenderer): Filter = Filter { next: HttpHandler ->
    { request ->
        val response = next(request)
        if (response.status.successful) {
            response
        } else {
            response.body(renderer(ShowErrorInfoViewModel(request.uri)))
        }
    }
}

fun main() {
    val schedule = createSchedule()
    val renderer = PebbleTemplates().HotReload("src/main/resources")
    val htmlView = Body.viewModel(renderer, ContentType.TEXT_HTML).toLens()
    val printingApp: HttpHandler = PrintRequest()
        .then(showErrorMessageFilter(renderer))
        // .then(ServerFilters.CatchAll())
        .then(app(renderer, htmlView, schedule))

    val server = printingApp.asServer(Undertow(9000)).start()
    println("Server started on http://localhost:" + server.port())
}
